package com.zhentao.information.cache;

import io.netty.channel.Channel;
import io.netty.channel.ChannelHandlerContext;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * Channel缓存管理类
 * 用于管理用户Channel
 */
@Slf4j
@Component
public class ChannelCache {
    
    /**
     * 用户ID和Channel的映射关系
     * key: 用户ID
     * value: Channel上下文
     */
    private static final Map<String, ChannelHandlerContext> USER_CHANNEL_MAP = new ConcurrentHashMap<>();

    private final Map<ChannelHandlerContext, String> reverseMap = new ConcurrentHashMap<>();

    /**
     * 添加用户Channel映射
     * @param userId 用户ID
     * @param ctx Channel上下文
     */
    public void addCache(String userId, ChannelHandlerContext ctx) {
        USER_CHANNEL_MAP.put(userId, ctx);
        reverseMap.put(ctx, userId);
        log.info("用户 {} 的Channel已缓存", userId);
    }

    /**
     * 获取用户的Channel
     * @param userId 用户ID
     * @return Channel上下文
     */
    public ChannelHandlerContext getCache(String userId) {
        return USER_CHANNEL_MAP.get(userId);
    }

    /**
     * 移除用户Channel映射
     * @param ctx Channel上下文
     */
    public void removeCache(ChannelHandlerContext ctx) {
        String userId = reverseMap.remove(ctx);
        if (userId != null) {
            USER_CHANNEL_MAP.remove(userId);
            log.info("用户 {} 的Channel已移除", userId);
        }
    }

    /**
     * 移除用户Channel映射
     * @param userId 用户ID
     */
    public void removeCache(String userId) {
        ChannelHandlerContext ctx = USER_CHANNEL_MAP.remove(userId);
        if (ctx != null) {
            reverseMap.remove(ctx);
            log.info("用户 {} 的Channel已移除", userId);
        }
    }

    /**
     * 获取所有用户Channel映射
     * @return 用户Channel映射Map
     */
    public Map<String, ChannelHandlerContext> getAllCache() {
        return USER_CHANNEL_MAP;
    }

    /**
     * 判断用户是否在线
     * @param userId 用户ID
     * @return 是否在线
     */
    public boolean isOnline(String userId) {
        return USER_CHANNEL_MAP.containsKey(userId);
    }
} 