package com.zhentao.user.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zhentao.tool.TokenUtils;
import com.zhentao.user.domain.UserLocation;
import com.zhentao.user.domain.UserLogin;
import com.zhentao.user.domain.UserOnlineStatus;
import com.zhentao.user.dto.LocationDto;
import com.zhentao.user.dto.NearbyUserDTO;
import com.zhentao.user.mapper.UserLocationMapper;
import com.zhentao.user.mapper.UserLoginMapper;
import com.zhentao.user.mapper.UserOnlineStatusMapper;
import com.zhentao.user.service.UserLocationService;
import com.zhentao.user.service.UserOnlineStatusService;
import com.zhentao.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.data.geo.*;
import org.springframework.data.redis.connection.RedisGeoCommands;
import org.springframework.data.redis.core.RedisCallback;
import org.springframework.data.redis.core.RedisOperations;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.SessionCallback;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.*;
import java.util.stream.Collectors;

/**
* @author 31810
* @description 针对表【user_location(用户位置记录表)】的数据库操作Service实现
* @createDate 2025-06-18 15:27:28
*/
@Service
public class UserLocationServiceImpl extends ServiceImpl<UserLocationMapper, UserLocation>
    implements UserLocationService{

    @Autowired
    private RedisTemplate<String,String>redisTemplate;
    @Autowired
    private UserLoginMapper userLoginMapper;
    @Autowired
    private UserOnlineStatusMapper userOnlineStatusMapper;
    @Autowired
    private UserLocationMapper userLocationMapper;
    @Autowired
    private DataSource dataSource;

    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private UserOnlineStatusService userOnlineStatusService;
    private static final String KEY_USER_LOCATIONS = "user:locations";
    private static final String KEY_ONLINE_USERS = "online_users";
    private static final String KEY_USER_STATUS = "user_status:%s";
    public boolean insertLocation(Long userId, String wktLocation) {
        String sql = "INSERT INTO user_location (user_id, location, update_time) " +
                "VALUES (?, ST_GeomFromText(?), ?) " +
                "ON DUPLICATE KEY UPDATE location = ST_GeomFromText(?), update_time = ?";

        try (Connection conn = dataSource.getConnection();
             PreparedStatement ps = conn.prepareStatement(sql)) {

            // 设置INSERT部分的参数 (3个)
            ps.setLong(1, userId);
            ps.setString(2, wktLocation);
            ps.setTimestamp(3, new java.sql.Timestamp(new Date().getTime()));

            // 设置UPDATE部分的参数 (2个)
            ps.setString(4, wktLocation);
            ps.setTimestamp(5, new java.sql.Timestamp(new Date().getTime()));

            int affectedRows = ps.executeUpdate();
            return affectedRows > 0;

        } catch (SQLException e) {
            e.printStackTrace();
            return false;
        }
    }

    @Override
    public Result updateUserLocation(LocationDto locationDto, String token) {
        // 从Token中获取用户ID
        String userIdFromToken = TokenUtils.getUserIdFromToken(token);
        Long userId = Long.parseLong(userIdFromToken);

        // 处理经纬度精度（统一为6位小数）
        double longitude = Math.round(locationDto.getLongitude() * 1000000.0) / 1000000.0;
        double latitude = Math.round(locationDto.getLatitude() * 1000000.0) / 1000000.0;

        // 验证经纬度有效性
        if (longitude < -180 || longitude > 180 || latitude < -90 || latitude > 90) {
            return Result.ERR(null, "经纬度必须在经度[-180, 180]和纬度[-90, 90]范围内");
        }

        // 构建统一的WKT格式（与数据库一致）
        String wktLocation = String.format("POINT(%.6f %.6f)", longitude, latitude);

        try {
            // 在事务中同步更新Redis和MySQL
            updateLocationInTransaction(userId, wktLocation, longitude, latitude);
            return Result.OK(null, "更新位置成功");
        } catch (Exception e) {
            log.error("更新用户位置失败");
            return Result.ERR(null, "更新位置失败，请稍后重试");
        }
    }

    @Transactional
    public void updateLocationInTransaction(Long userId, String wktLocation, double longitude, double latitude) {
        QueryWrapper<UserLocation> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("user_id", userId);
        UserLocation existingLocation = userLocationMapper.selectOne(queryWrapper);

        // 2. 根据存在性决定是插入还是更新
        if (existingLocation != null) {
            // 更新现有记录
            userLocationMapper.updateByUserId(wktLocation, new Date(), userId);
        } else {
            // 插入新记录
            this.insertLocation(userId, wktLocation);
        }
        // 2. 更新Redis中的用户位置（确保与MySQL数据一致）
        redisTemplate.opsForGeo().add(
                KEY_USER_LOCATIONS,
                new Point(longitude, latitude),
                userId.toString()
        );

        // 3. 更新用户在线状态
        userOnlineStatusService.userGoOnline(userId);
    }

    @Override
    public Result getNearbyUsers(LocationDto locationDto,String token) {
        String strUid=TokenUtils.getUserIdFromToken(token);
        if(strUid==null){
            return Result.ERR(null, "token无效");
        }
        List<Point> points = redisTemplate.opsForGeo().position(KEY_USER_LOCATIONS, strUid);
        locationDto.setLongitude(points.get(0).getX());
        locationDto.setLatitude(points.get(0).getY());
        System.out.println("查询参数：longitude=" + locationDto.getLongitude() +
                ", latitude=" + locationDto.getLatitude() +
                ", radius=" + locationDto.getRadius() +
                ", limit=" + locationDto.getLimit());
        GeoResults<RedisGeoCommands.GeoLocation<String>>results=redisTemplate.opsForGeo().radius(
                KEY_USER_LOCATIONS,
                new Circle(new Point(locationDto.getLongitude(),locationDto.getLatitude()),new Distance(locationDto.getRadius(), Metrics.KILOMETERS)),
                RedisGeoCommands.GeoRadiusCommandArgs.newGeoRadiusArgs().includeDistance().includeCoordinates().limit(locationDto.getLimit()));

        List<NearbyUserDTO> nearbyUsers = new ArrayList<>();
        if (results == null) {
            return Result.OK(nearbyUsers, "未找到附近的用户");
        }
        List<Long> userIds = new ArrayList<>();
        Map<String, GeoResult<RedisGeoCommands.GeoLocation<String>>> userIdToResult = new HashMap<>();
        for (GeoResult<RedisGeoCommands.GeoLocation<String>> result:results) {
            if (result == null||results.getContent().isEmpty()) {
                continue; // 跳过无效结果
            }
            String userIdStr=result.getContent().getName();
            Long userId=Long.parseLong(userIdStr);
            userIds.add(userId);
            userIdToResult.put(userIdStr,result);
        }
        System.err.println("查到"+userIds.size()+"个用户");
        if(userIds.size()==0){
            return Result.OK(nearbyUsers, "未找到附近的用户");
        }
        List<UserLogin> userList = userLoginMapper.selectBatchIds(userIds);
        Map<Long, UserLogin> userMap = userList.stream()
                .collect(Collectors.toMap(UserLogin::getId, user -> user));
        //批量验证用户在线状态
        Map<Long, Boolean> onlineStatusMap = isUserOnlineBatch(userIds);
        System.err.println("查到"+onlineStatusMap.size()+"个用户在线状态");
        System.err.println(onlineStatusMap);
        for (Map.Entry<String, GeoResult<RedisGeoCommands.GeoLocation<String>>> entry : userIdToResult.entrySet()) {
            String userIdStr = entry.getKey();
            GeoResult<RedisGeoCommands.GeoLocation<String>> result = entry.getValue();
            try {
                Long userId = Long.parseLong(userIdStr);
                Distance distance = result.getDistance();
                UserLogin user = userMap.get(userId);

                if (user != null) {
                    NearbyUserDTO userDTO = new NearbyUserDTO();
                    userDTO.setUserId(userId);
                    userDTO.setUsername(user.getUserUsername());
                    userDTO.setAvatarUrl(user.getAvatar() != null ? user.getAvatar() : "");

                    // 处理距离
                    if (distance != null) {
                        userDTO.setDistance(distance.getValue());
                        // 将单位缩写转换为中文（如 "KM" -> "公里"）
                        userDTO.setDistanceUnit(convertDistanceUnit(distance.getMetric().getAbbreviation()));
                    } else {
                        userDTO.setDistance(0);
                        userDTO.setDistanceUnit("公里");
                    }

                    userDTO.setOnline(onlineStatusMap.getOrDefault(userId, false));

                    // 处理坐标
                    Point point = result.getContent().getPoint();
                    boolean isMyself=false;
                    if(userDTO.getUserId().equals(Long.parseLong(strUid))){
                        isMyself=true;
                    }
                    if (point != null) {
                        userDTO.setLocation(new NearbyUserDTO.Location(point.getX(), point.getY(),isMyself));
                    }

                    nearbyUsers.add(userDTO);
                }
            } catch (Exception e) {
                log.error("组装附近用户数据时出错", e);
            }
        }
        return Result.OK(nearbyUsers, nearbyUsers.isEmpty() ? "未找到附近的用户" : "查询成功");
    }

    private String convertDistanceUnit(String abbreviation) {
        if ("KM".equals(abbreviation)) {
            return "公里";
        } else if ("M".equals(abbreviation)) {
            return "米";
        }
        return abbreviation;
    }


    public Map<Long, Boolean> isUserOnlineBatch(List<Long> userIds) {
        Map<Long, Boolean> resultMap = new HashMap<>();

        List<Object> results = redisTemplate.executePipelined((RedisCallback<Object>) connection -> {
            for (Long userId : userIds) {
                connection.sIsMember(KEY_ONLINE_USERS.getBytes(), userId.toString().getBytes());
                String statusKey = String.format(KEY_USER_STATUS, userId);
                connection.hGet(statusKey.getBytes(), "last_heartbeat".getBytes());
            }
            return null;
        });

        for (int i = 0; i < userIds.size(); i++) {
            Long userId = userIds.get(i);
            Boolean isOnline = (Boolean) results.get(i * 2);
            byte[] lastHeartbeatBytes = (byte[]) results.get(i * 2 + 1);

            boolean onlineStatus = false;
            if (Boolean.TRUE.equals(isOnline) && lastHeartbeatBytes != null) {
                long lastHeartbeat = Long.parseLong(new String(lastHeartbeatBytes));
                onlineStatus = System.currentTimeMillis() - lastHeartbeat < 5 * 60 * 1000;
            }

            resultMap.put(userId, onlineStatus);
        }

        return resultMap;
    }
    //更新用户在线状态
    @Scheduled(cron = "0 * * * * *")
    public void cleanOffineUsers() {
        System.err.println("执行清理离线用户任务...");
        Set<String> onlineUserIds = redisTemplate.opsForSet().members(KEY_ONLINE_USERS);
        if (onlineUserIds == null || onlineUserIds.isEmpty()) {
            return;
        }

        long currentTime = System.currentTimeMillis();
        List<Object> results = redisTemplate.execute(new SessionCallback<List<Object>>() {
            @Override
            public List<Object> execute(RedisOperations operations) throws DataAccessException {
                operations.multi();
                for (String onlineStrId : onlineUserIds) {
                    String cleanedId = onlineStrId.replaceAll("\"", "");
                    Long userId = Long.valueOf(cleanedId);
                    String statusKey = String.format(KEY_USER_STATUS, userId);
                    operations.opsForHash().get(statusKey, "last_heartbeat");
                }
                return operations.exec();
            }
        });

        if (results != null) {
            System.err.println("清理离线用户任务执行结果：" + results.size());
            int i = 0;
            for (String onlineStrId : onlineUserIds) {
                Object lastHeartbeatObj = results.get(i++);
                System.err.println(lastHeartbeatObj);
                if (lastHeartbeatObj != null) {
                    long lastHeartbeat = Long.parseLong(lastHeartbeatObj.toString());
                    System.err.println(lastHeartbeat);
                    if (currentTime - lastHeartbeat > 5 * 60 * 1000) {
                        QueryWrapper<UserOnlineStatus> queryWrapper = new QueryWrapper<>();
                        queryWrapper.eq("user_id", Long.valueOf(onlineStrId));
                        UserOnlineStatus onlineStatus = userOnlineStatusMapper.selectOne(queryWrapper);
                        onlineStatus.setIsOnline(0);
                        userOnlineStatusMapper.updateById(onlineStatus);
                        System.err.println("用户 " + onlineStrId + " 已离线，更新状态为不在线");
                        redisTemplate.opsForSet().remove(KEY_ONLINE_USERS, onlineStrId);
                    }
                }
            }
        }
    }
}




